
/**
 * ApiEx接口信息, 需与原生代码一致, 由Android开发人员维护
 */
var ApiEx = {
    VERSION: "2.0"
};


/**
 * Js于本地Java代码交互逻辑, 使用了Js中的prompt输入对话框函数, 在Java中拦截, 然后返回结果
 * 接口机制:
 *      Java中通过拦截 'orange://Type/Method?QueryString' 格式的message(经过encode处理), 触发调用本地代码的逻辑, 其中:
 *          Type:           扩展类型名称, 例如, Toast
 *          Method:         扩展方法名, 例如, makeText
 *          QueryString:    传入参数, 例如, text=Hello Me&duration=0, 只支持整型, 字符串, 布尔值
 *
 *     Java运行结果通过prompt获得, 然后返回其eval()之后的Js值, 支持整型, 字符串, 布尔值, null, 简单Key-Value Object
 *
 *     如果没有指定合法的接口, 返回undefined
 *
 *     对于异步:
 *          接口是否异步, 取决于Java的实现
 *          通过在QueryString中加入 callback=your_callback_function 以获得回调触发
 *
 * 使用prompt机制的原因:
 *      a) 基于addJavascriptInterface()的Js注入存在高安全风险
 *      b) 基于Js的单线程机制, 通过url拦截的方式难以实现同步函数
 *
 * @param type 扩展类型, 例如, Toast
 * @param method 扩展方法, 例如, makeText
 * @param argNames 参数名, 例如, ['text', 'duration']
 * @param argValues 对应参数值, 例如, ['Hello Me', 0]
 * 
 * returns 返回值, int, string, bool, null, 简单Key-Value Object, 如果接口不合法, 返回undefined
 */
var exec = function (type, method, argNames, argValues) {
    var request = 'orange://' + type + '/' + method + '?';
    for (var i = 0; i < argNames.length; i++) {
        if (argValues[i] != null) {
            request += argNames[i] + '=' + encodeURIComponent(argValues[i]) + '&';
        }
    }
    request = encodeURI(request);

    console.log('ApiEx$request ApiEx with: ' + request);
    var result = undefined;
    eval('var result = ' + prompt(request) + ';');
    console.log('ApiEx$result = ' + result);

    return result;
};

/**
 * 回调控制器
 * 将回调方法通过addCallback加到控制器, 返回值为方法管理key
 * 将key传递给exec的回调参数
 *
 * ps. 对于inner function, 不能直接从外部传递调用参数
 *     对于这种情况下的回调函数, 可在函数体中同过this.params获得回调参数列表
 *
 */
var CallbackController = {
	callbacks: {},
	params: {},

	addCallback: function (callback) {
		if (typeof callback != 'function') {
            console.warn('ApiEx$addCallback without a function');
			return null;
		}
		var key = Math.random().toString(16);
		CallbackController.callbacks[key] = callback;
		CallbackController.params[key] = [];
		console.log('ApiEx$addCallback, with key: ' + key);
		return key;
	},

	addParam: function(key, param) {
        if (CallbackController.params[key] != null) {
            CallbackController.params[key].push(param);
        }
		console.log('ApiEx$addParam to function ' + key + ', with (' + param + ')');
	},

	invoke: function(key) {
		if (CallbackController.callbacks[key] != null) {
            console.log('ApiEx$invoke callback: ' + key);
			var paramString = '';
			var paramCount = CallbackController.params[key].length;
			for (var i = 0; i < paramCount; i++) {
				paramString += ", CallbackController.params['" + key + "'][" + i + "]";
			}

			var js = "var obj = {}; obj.key = '" + key + "'; obj.params = CallbackController.params['" + key + "']; ";
			js += "CallbackController.callbacks['" + key + "'].call(obj" + paramString + ");";
			eval(js);
            CallbackController.params[key] = [];
		}
	}

};

/**
 * Toast 扩展
 */
Toast = {

    LENGTH_SHORT: 0,
    LENGTH_LONG: 1,

    /**
     * 2.0+
     * Toast提示
     * @param text 提示文本
     * @param duration 提示时长, LENGTH_SHORT(default), LENGTH_LONG
     */
    makeText: function (text, duration) {
        if (duration == null) {
            duration = Toast.LENGTH_SHORT;
        }
        return exec('Toast', 'makeText', ['text', 'duration'], [text, duration]);
    }

};

/**
 * TitleBar 扩展, 控制标题导航栏
 */
TitleBar = {

    /**
     * 2.0+
     * 设置标题
     * @param title 标题字符串
     */
    setTitle: function(title) {
        return exec('TitleBar', 'setTitle', ['title'], [title]);
    },

    /**
     * 2.0+
     * 设置标题颜色
     * @param colorString 颜色值字符串, #AARRGGBB, #RRGGBB, null设为默认(BLACK)
     */
    setTitleColor: function (colorString) {
        return exec('TitleBar', 'setTitleColor', ['colorString'], [colorString]);
    },

    /**
     * 2.0+
     * 设置标题栏背景图片
     * @param imgFile 图片绝对路径, 只支持本地文件
     */
    setBackgroundImage: function(imgFile) {
        return exec('TitleBar', 'setBackgroundImage', ['imgFile'], [imgFile]);
    },

    /**
     * 2.0+
     * 设置标题栏背景颜色
     * @param colorString 颜色值字符串, #AARRGGBB, #RRGGBB, null设为默认(WHITE)
     */
    setBackgroundColor: function(colorString) {
        return exec('TitleBar', 'setBackgroundColor', ['colorString'], [colorString]);
    },

    /**
     * 2.0+
     * 设置标题右菜单图标
     *      如果imgFile和text均为null, 则菜单被自动设为隐藏状态
     *      否则, 菜单被自动设置为可见
     * @param imgFile 图标绝对路径, 如果为null, 则显示文本
     * @param text 说明文本, 图标为null, 则显示在标题菜单
     * @param callback 点击回调函数, 回调无参数, 如果imgFile和text均为null, 则被忽略
     */
    setRightItem: function(imgFile, text, callback) {
        var callback_key = CallbackController.addCallback(callback);
        return exec('TitleBar', 'setRightItem', ['imgFile', 'text', 'callback'], [imgFile, text, callback_key]);
    },

    /**
     * 2.0+
     * 设置有菜单文字颜色
     * @param colorString 颜色值字符串, #AARRGGBB, #RRGGBB, null设为默认(BLACK)
     */
    setRightItemTextColor: function(colorString) {
        return exec('TitleBar', 'setRightItemTextColor', ['colorString'], [colorString]);
    },

    /**
     * 2.0+
     * 设置右菜单可见性, 如果菜单的imgFile和text均为null, 则设置值无效且菜单保持为隐藏状态
     * @param isVisible true or false
     */
    setRightItemVisible: function(isVisible) {
        return exec('TitleBar', 'setRightItemVisible', ['isVisible'], [isVisible]);
    }

};

/**
 * Preference 扩展, 对应与Android的Preference接口
 */

Preference = {
    /**
     * 2.0+
     * 写入数据, 值以string方式表示
     * @param key 数据字段
     * @param value 数据值
     * @param domain 数据所属域, 默认为DEFAULT
     */
    putValue: function(key, value, domain) {
        return exec('Preference', 'putValue', ['key', 'value', 'domain'], [key, value, domain]);
    },

    /**
     * 2.0+
     * 读取数据, 值以string方式表示
     * @param key 数据字段
     * @param domain 数据所属域, 默认为DEFAULT
     * @return 返回value值 或null
     */
    getValue: function(key, domain) {
        return exec('Preference', 'getValue', ['key', 'domain'], [key, domain]);
    }

};

/**
 * HttpClient 扩展, 用于网络访问
 */
HttpClient = {
    /**
     * 2.0+
     * 网络访问, 结果以string类型返回到回调函数
     * @param url 地址
     * @param method HEAD, GET, POST, PUT, DELETE
     * @param timeout 超时时长, 毫秒, 默认为30000
     * @param header http头部, js object 类型
     * @param data http负载, 字符串类型
     * @param succeed 成功回调, 回调参数为(code, message)
     * @param failed 失败回到, 回调参数为(code, message), 皆为字符串类型
     * @param noCache 不使用缓存, null则使用全局设置, 默认为false
     */
    request: function (url, method, timeout, headers, data, succeed, failed, noCache) {
		var succeed_key = CallbackController.addCallback(succeed);
		var failed_key = CallbackController.addCallback(failed);
		if (headers != null) {
		    headers = JSON.stringify(headers);
        }
		return exec('HttpClient', 'request', ['url', 'method', 'timeout', 'headers', 'data', 'succeed', 'failed', 'noCache'],
                [url, method, timeout, headers, data, succeed_key, failed_key, noCache]);
	},

    /**
     * 2.0+
     * 上传图片到指定服务器
     * @param key 图片文件key
     * @param url 上传地址
     * @param compress 压缩比率, 0-100
     * @param succeed 成功回调函数, ()
     * @param failed 失败回调函数, (code, message)
     * @param progress 进度回调函数, (progress)
     */
    uploadImage: function (key, url, compress, succeed, failed, progress) {
        var succeed_key = CallbackController.addCallback(succeed);
        var failed_key = CallbackController.addCallback(failed);
        var progress_key = CallbackController.addCallback(progress);
        return exec('HttpClient', 'uploadImage', ['key', 'url', 'compress', 'succeed', 'failed', 'progress'],
                [key, url, compress, succeed_key, failed_key, progress_key]);
    }
};

/**
 * Photo 扩展, 用于选择图片或拍照
 */
Photo = {
    /**
     * 2.0+
     * 选取图片
     * @param onSucceed 成功回调, 回调参数:
     *                  key: 图片引用key, 用于访问图片, js不能直接获取图片的路径信息
     *                  width: 图片宽度
     *                  height: 图片高度
     *                  size: 图片大小, 单位字节
     * @param onCanceled 取消回调, 无回调参数
     * @param onError 错误回调, 回调参数:
     *                  code: 错误代码
     *                  message: 错误描述
     */
    pick: function (onSucceed, onCanceled, onError) {
        var onSucceed_key = CallbackController.addCallback(onSucceed);
        var onCanceled_key = CallbackController.addCallback(onCanceled);
        var onError_key = CallbackController.addCallback(onError);
        return exec('Photo', 'pick', ['onSucceed', 'onCanceled', 'onError'],
            [onSucceed_key, onCanceled_key, onError_key]);
    },

    /**
     * 2.0+
     * 拍照
     * @param onSucceed 成功回调, 回调参数:
     *                  key: 图片引用key, 用于访问图片, js不能直接获取图片的路径信息
     *                  width: 图片宽度
     *                  height: 图片高度
     *                  size: 图片大小, 单位字节
     * @param onCanceled 取消回调, 无回调参数
     * @param onError 错误回调, 回调参数:
     *                  code: 错误代码
     *                  message: 错误描述
     */
    capture: function(onSucceed, onCanceled, onError) {
        var onSucceed_key = CallbackController.addCallback(onSucceed);
        var onCanceled_key = CallbackController.addCallback(onCanceled);
        var onError_key = CallbackController.addCallback(onError);
        return exec('Photo', 'capture', ['onSucceed', 'onCanceled', 'onError'],
            [onSucceed_key, onCanceled_key, onError_key]);
    }

}


